/*******************************************************************************
 * Copyright (c) 2004 Elias Volanakis.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*Elias Volanakis - initial API and implementation
 *    Chris Aniszczyk - updated to work with EMF
 *    IBM Corporation
*******************************************************************************/

package org.eclipse.gef.examples.shapes.emf.model.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.examples.shapes.emf.model.Shape;
import org.eclipse.gef.examples.shapes.emf.model.ShapesDiagram;
import org.eclipse.gef.requests.CreateRequest;


/**
 * A command to add a Shape to a ShapeDiagram.
 * The command can be undone or redone.
 * @author Elias Volanakis
 * @author Chris Aniszczyk - updated for EMF
 */
public class ShapeCreateCommand extends Command {
/** The new shape. */ 
private Shape newShape;

/** ShapeDiagram to add to. */
private final ShapesDiagram parent;
/** A request to create a new Shape. */
private final CreateRequest request;
/** True, if newShape was added to parent. */
private boolean shapeAdded;

/**
 * Create a command that will add a new Shape to a ShapesDiagram.
 * @param parent the ShapesDiagram that will hold the new element
 * @param req     a request to create a new Shape
 * @throws IllegalArgumentException if any parameter is null, or the request
 * 						  does not provide a new Shape instance
 */
public ShapeCreateCommand(ShapesDiagram parent, CreateRequest req) {
	if (parent == null || req == null || !(req.getNewObject() instanceof Shape)) {
		throw new IllegalArgumentException();
	}
	this.parent = parent;
	this.request = req;
	setLabel("shape creation");
}

/* (non-Javadoc)
 * @see org.eclipse.gef.commands.Command#canUndo()
 */
public boolean canUndo() {
	return shapeAdded;
}

/* (non-Javadoc)
 * @see org.eclipse.gef.commands.Command#execute()
 */
public void execute() {
	// Obtain the new Shape instance from the request.
	// This causes the factory stored in the request to create a new instance.
	// The factory is supplied in the palette-tool-entry, see
	// ShapesEditorPaletteFactory#createComponentsGroup()
	newShape = (Shape) request.getNewObject();
	// Get desired location and size from the request
	if(request.getSize() != null) { // we'll use default size if it's null
		newShape.setHeight(request.getSize().height);
		newShape.setWidth(request.getSize().width);
	}
	newShape.setX(request.getLocation().x);
	newShape.setY(request.getLocation().y);
	redo();
}

/* (non-Javadoc)
 * @see org.eclipse.gef.commands.Command#redo()
 */
public void redo() {
	shapeAdded = parent.getShapes().add(newShape);
}

/* (non-Javadoc)
 * @see org.eclipse.gef.commands.Command#undo()
 */
public void undo() {
	parent.getShapes().remove(newShape);
}
	
}
